searchData={"items":[{"type":"module","title":"ExWal","doc":"**ExWal is a project that aims to provide a solution for managing write-ahead log (WAL) in Elixir.**\n\nExWal is a GenServer-based module that provides functionality for working with a Write-Ahead Log (WAL). \n\nThe module includes utilities for managing WAL files, segments, blocks, and entries. It also supports options for customizing the WAL path, synchronization behavior, segment size, segment cache size, and the underlying store module.\n\nFor more information on how to use ExWal, refer to the [documentation](https://hex.pm/docs/publish) and examples.","ref":"ExWal.html"},{"type":"module","title":"Installation - ExWal","doc":"The package can be installed by adding `ex_wal` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_wal, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"ExWal.html#module-installation"},{"type":"module","title":"Design - ExWal","doc":"This project has heavily borrowed the code experience from the project at [tidwall/wal](https://github.com/tidwall/wal). In terms of design, WAL packages log entries into individual segments for maintenance. I take the last segment being written as the \"hot\" part, and while writing hot data, corresponding files will be appended. When the volume of \"hot\" data exceeds the configured value, we will transfer the hot data to the \"cold\" part and reopen a new \"hot\" segment.\n\nIn the process of reading, in order to avoid frequent opening of cold data, I added an LRU cache to accelerate the reading speed of local data.\n\nIn the operations of maintaining Segment and corresponding log entry, there are a considerable number of operations that find field based on index. However, the performance of [List](https://hexdocs.pm/elixir/List.html) provided by Elixir is not ideal in this situation, so we choose to use the [array](https://www.erlang.org/doc/man/array) module of erlang to store segment and block.","ref":"ExWal.html#module-design"},{"type":"module","title":"Usage - ExWal","doc":"```elixir\nSupervisor.start_link(\n  [\n    {ExWal,\n     [\n       name: :wal_test,\n       path: \"/tmp/wal_test\",\n       nosync: false,\n       # 4k per segment\n       segment_size: 4 * 1024,\n       # cache max 5 segments\n       segment_cache_size: 5\n     ]}\n  ],\n  strategy: :one_for_one\n)\n\nlatest = ExWal.last_index(:wal_test)\nLogger.info(\"latest: #{latest}\")\n\n# write 10k entries\nentries =\n  Enum.map((latest + 1)..(latest + 10_000), fn i -> Entry.new(i, \"Hello Elixir #{i}\") end)\n\n:ok = ExWal.write(:wal_test, entries)\n\nlatest = ExWal.last_index(:wal_test)\nLogger.info(\"latest: #{latest}\") # should be latest + 10_000\n\n# read\n{:ok, ret} = ExWal.read(:wal_test, latest - 10)\nLogger.info(\"idx: #{latest - 10}, content: #{ret}\")\n\n# truncate before\n:ok = ExWal.truncate_before(:wal_test, latest - 100)\nfirst = ExWal.first_index(:wal_test)\nLogger.info(\"first: #{first}\") # should be latest - 100\n\n# truncate after\n:ok = ExWal.truncate_after(:wal_test, latest - 5)\nlatest = ExWal.last_index(:wal_test)\nLogger.info(\"latest: #{latest}\") # should be latest - 5\n```","ref":"ExWal.html#module-usage"},{"type":"module","title":"Benchmark - ExWal","doc":"STILL IN PROGRESS\n\n-----\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"ExWal.html#module-benchmark"},{"type":"function","title":"ExWal.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExWal.html#child_spec/1"},{"type":"function","title":"ExWal.clear/1","doc":"Clears the write-ahead log (WAL) by removing all entries.","ref":"ExWal.html#clear/1"},{"type":"function","title":"Examples - ExWal.clear/1","doc":"iex> MyApp.WAL.clear(:my_wal)\n    :ok","ref":"ExWal.html#clear/1-examples"},{"type":"function","title":"Parameters - ExWal.clear/1","doc":"* `name_or_pid` - The name or process identifier of the WAL GenServer.","ref":"ExWal.html#clear/1-parameters"},{"type":"function","title":"Returns - ExWal.clear/1","doc":"- `:ok` - If the clearing is successful.","ref":"ExWal.html#clear/1-returns"},{"type":"function","title":"ExWal.first_index/1","doc":"Get the first index of WAL","ref":"ExWal.html#first_index/1"},{"type":"function","title":"ExWal.last_index/1","doc":"Get the last index of WAL","ref":"ExWal.html#last_index/1"},{"type":"function","title":"ExWal.read/3","doc":"Read entry content from WAL by index. if index is not found, return {:error, :index_not_found}","ref":"ExWal.html#read/3"},{"type":"function","title":"Examples - ExWal.read/3","doc":"iex> {:ok, data} = ExWal.read(:wal_name, 1)","ref":"ExWal.html#read/3-examples"},{"type":"function","title":"ExWal.segment_count/1","doc":"Get the segment count of WAL","ref":"ExWal.html#segment_count/1"},{"type":"function","title":"ExWal.start_link/1","doc":"Start a WAL process","ref":"ExWal.html#start_link/1"},{"type":"function","title":"Options - ExWal.start_link/1","doc":"* `:path` (`t:String.t/0`) - The path of the WAL. Default is `./wal`. The default value is `\"./wal\"`.\n\n* `:nosync` (`t:boolean/0`) - If true, WAL will not sync to disk. Default is false. The default value is `false`.\n\n* `:segment_size` (`t:integer/0`) - The size of each segment file. Default is 16MB. The default value is `16777216`.\n\n* `:segment_cache_size` (`t:integer/0`) - The size of the segment cache. Default is 100. The default value is `2`.\n\n* `:store` (`t:atom/0`) - The store module which implement `ExWal.Store`. Default is `ExWal.Store.File`. The default value is `ExWal.Store.File`.\n\n* `:store_opts` (`t:keyword/0`) - The options for the store module. The default value is `[]`.\n\n* `:dir_permission` (`t:integer/0`) - The permission of the directory. Default is 0o750. The default value is `488`.\n\n* `:file_permission` (`t:integer/0`) - The permission of the file. Default is 0o640. The default value is `416`.\n\n* `:name` (`t:atom/0`) - The name of the WAL. Default is `wal`. The default value is `ExWal`.","ref":"ExWal.html#start_link/1-options"},{"type":"function","title":"ExWal.stop/1","doc":"Stop a WAL process","ref":"ExWal.html#stop/1"},{"type":"function","title":"ExWal.truncate_after/3","doc":"Truncates the write-ahead log (WAL) after a specific index.","ref":"ExWal.html#truncate_after/3"},{"type":"function","title":"Examples - ExWal.truncate_after/3","doc":"iex> MyApp.WAL.truncate_after(:my_wal, 10)\n    :ok","ref":"ExWal.html#truncate_after/3-examples"},{"type":"function","title":"Parameters - ExWal.truncate_after/3","doc":"* `name_or_pid` - The name or process identifier of the WAL GenServer.\n  * `index` - The index after which to truncate the WAL.\n  * `timeout` (optional) - The timeout value in milliseconds (default: 5000).","ref":"ExWal.html#truncate_after/3-parameters"},{"type":"function","title":"Returns - ExWal.truncate_after/3","doc":"- `:ok` - If the truncation is successful.\n- `{:error, :index_out_of_range}` - If the provided index is out of range.","ref":"ExWal.html#truncate_after/3-returns"},{"type":"function","title":"ExWal.truncate_before/3","doc":"Truncates the write-ahead log (WAL) before a specific index.","ref":"ExWal.html#truncate_before/3"},{"type":"function","title":"Examples - ExWal.truncate_before/3","doc":"iex> MyApp.WAL.truncate_before(:my_wal, 10)\n    :ok","ref":"ExWal.html#truncate_before/3-examples"},{"type":"function","title":"Parameters - ExWal.truncate_before/3","doc":"* `name_or_pid` - The name or process identifier of the WAL GenServer.\n  * `index` - The index before which to truncate the WAL.\n  * `timeout` (optional) - The timeout value in milliseconds (default: 5000).","ref":"ExWal.html#truncate_before/3-parameters"},{"type":"function","title":"Returns - ExWal.truncate_before/3","doc":"- `:ok` - If the truncation is successful.\n- `{:error, :index_out_of_range}` - If the provided index is out of range.","ref":"ExWal.html#truncate_before/3-returns"},{"type":"function","title":"ExWal.write/3","doc":"Write entries to WAL, the entries must be strictly consecutive and incremental,\nand the index of the first entry must be WAL's last_index + 1.","ref":"ExWal.html#write/3"},{"type":"function","title":"Examples - ExWal.write/3","doc":"iex> last_index = ExWal.last_index(:wal_name)\n    iex> enties = last_index..(last_index+10) |> Enum.map(fn i -> Entry.new(i, \"some data\") end)\n    iex> :ok = ExWal.write(:wal_name, entries)","ref":"ExWal.html#write/3-examples"},{"type":"type","title":"ExWal.index/0","doc":"","ref":"ExWal.html#t:index/0"},{"type":"type","title":"ExWal.t/0","doc":"","ref":"ExWal.html#t:t/0"},{"type":"type","title":"ExWal.wal_option_schema_t/0","doc":"","ref":"ExWal.html#t:wal_option_schema_t/0"},{"type":"module","title":"ExWal.Debug","doc":"Debug tools","ref":"ExWal.Debug.html"},{"type":"function","title":"ExWal.Debug.debug/1","doc":"","ref":"ExWal.Debug.html#debug/1"},{"type":"function","title":"ExWal.Debug.stacktrace/1","doc":"","ref":"ExWal.Debug.html#stacktrace/1"},{"type":"module","title":"ExWal.LRU","doc":"This modules implements a simple LRU cache, using 2 ets tables for it.\n\nFor using it, you need to start it:\n\n    iex> LRU.start_link(:my_cache, 1000)\n\nOr add it to your supervisor tree, like: `worker(LRU, [:my_cache, 1000])`","ref":"ExWal.LRU.html"},{"type":"module","title":"Using - ExWal.LRU","doc":"iex> LRU.start_link(:my_cache, 1000)\n    {:ok, #PID<0.60.0>}\n\n    iex> LRU.put(:my_cache, \"id\", \"value\")\n    :ok\n\n    iex> LRU.get(:my_cache, \"id\", touch = false)\n    \"value\"\n\nTo take some action when old keys are evicted from the cache when it is full,\nyou can pass an `:evict_fn` option to `LRU.start_link/3`. This is\nhelpful for cleaning up processes that depend on values in the cache, or\nlogging, or instrumentation of cache evictions etc.\n\n    iex> evict = fn(key,value) -> IO.inspect(\"#{key}=#{value} evicted\") end\n    iex> LRU.start_link(:my_cache, 10, evict_fn: evict)\n    {:ok, #PID<0.60.0>}","ref":"ExWal.LRU.html#module-using"},{"type":"module","title":"Design - ExWal.LRU","doc":"First ets table save the key values pairs, the second save order of inserted elements.","ref":"ExWal.LRU.html#module-design"},{"type":"function","title":"ExWal.LRU.clear/2","doc":"Removes all entries from cache.","ref":"ExWal.LRU.html#clear/2"},{"type":"function","title":"ExWal.LRU.delete/3","doc":"Removes the entry stored under the given `key` from cache.","ref":"ExWal.LRU.html#delete/3"},{"type":"function","title":"ExWal.LRU.get/4","doc":"Returns the `value` associated with `key` in `cache`. If `cache` does not contain `key`,\nreturns nil. `touch` defines, if the order in LRU should be actualized.","ref":"ExWal.LRU.html#get/4"},{"type":"function","title":"ExWal.LRU.handle_clear/1","doc":"","ref":"ExWal.LRU.html#handle_clear/1"},{"type":"function","title":"ExWal.LRU.put/4","doc":"Stores the given `value` under `key` in `cache`. If `cache` already has `key`, the stored\n`value` is replaced by the new one. This updates the order of LRU cache.","ref":"ExWal.LRU.html#put/4"},{"type":"function","title":"ExWal.LRU.select/3","doc":"Selects and returns the first value in the cache that matches the given condition.","ref":"ExWal.LRU.html#select/3"},{"type":"function","title":"Examples - ExWal.LRU.select/3","doc":"iex> LRU.select(:my_cache, fn value -> value > 10 end)\n    15\n\n#","ref":"ExWal.LRU.html#select/3-examples"},{"type":"function","title":"Parameters - ExWal.LRU.select/3","doc":"- `name` - The name of the cache.\n- `match_fun` - A function that takes a value and returns a boolean indicating whether it matches the condition.\n- `timeout` (optional) - The timeout value in milliseconds for the operation. Defaults to 5000.\n\n#","ref":"ExWal.LRU.html#select/3-parameters"},{"type":"function","title":"Returns - ExWal.LRU.select/3","doc":"The first value in the cache that matches the given condition, or `nil` if no match is found.","ref":"ExWal.LRU.html#select/3-returns"},{"type":"function","title":"ExWal.LRU.start_link/3","doc":"Creates an LRU of the given size as part of a supervision tree with a registered name","ref":"ExWal.LRU.html#start_link/3"},{"type":"function","title":"Options - ExWal.LRU.start_link/3","doc":"* `:evict_fn` - function that accepts (key, value) and takes some action when keys are\n    evicted when the cache is full.","ref":"ExWal.LRU.html#start_link/3-options"},{"type":"function","title":"ExWal.LRU.stop/1","doc":"","ref":"ExWal.LRU.html#stop/1"},{"type":"function","title":"ExWal.LRU.update/5","doc":"Updates a `value` in `cache`. If `key` is not present in `cache` then nothing is done.\n`touch` defines, if the order in LRU should be actualized. The function assumes, that\nthe element exists in a cache.","ref":"ExWal.LRU.html#update/5"},{"type":"type","title":"ExWal.LRU.t/0","doc":"","ref":"ExWal.LRU.html#t:t/0"},{"type":"module","title":"ExWal.Models.Block","doc":"A single block in a segment file.","ref":"ExWal.Models.Block.html"},{"type":"module","title":"Description - ExWal.Models.Block","doc":"The `ExWal.Models.Block` module represents a single block in a segment file.\nIt contains information about the offset and size of the block.","ref":"ExWal.Models.Block.html#module-description"},{"type":"module","title":"Examples - ExWal.Models.Block","doc":"iex> block = %ExWal.Models.Block{offset: 0, size: 1024}\n    %ExWal.Models.Block{offset: 0, size: 1024}","ref":"ExWal.Models.Block.html#module-examples"},{"type":"module","title":"Struct Fields - ExWal.Models.Block","doc":"- `offset` - The offset of the block within the segment file.\n- `size` - The size of the block in bytes.","ref":"ExWal.Models.Block.html#module-struct-fields"},{"type":"type","title":"ExWal.Models.Block.t/0","doc":"","ref":"ExWal.Models.Block.html#t:t/0"},{"type":"module","title":"ExWal.Models.Entry","doc":"entry struct for each record in WAL","ref":"ExWal.Models.Entry.html"},{"type":"function","title":"ExWal.Models.Entry.new/2","doc":"Create a new entry","ref":"ExWal.Models.Entry.html#new/2"},{"type":"function","title":"Examples - ExWal.Models.Entry.new/2","doc":"iex> Entry.new(1, \"Hello Elixir\")\n    %Entry{index: 1, data: \"Hello Elixir\"}","ref":"ExWal.Models.Entry.html#new/2-examples"},{"type":"type","title":"ExWal.Models.Entry.t/0","doc":"","ref":"ExWal.Models.Entry.html#t:t/0"},{"type":"module","title":"ExWal.Models.Segment","doc":"A single segment file.","ref":"ExWal.Models.Segment.html"},{"type":"module","title":"Description - ExWal.Models.Segment","doc":"The `ExWal.Models.Segment` module represents a single segment unit.\nIt contains information about the path, index, buffer, block count, and blocks within the segment.","ref":"ExWal.Models.Segment.html#module-description"},{"type":"module","title":"Examples - ExWal.Models.Segment","doc":"iex> segment = %ExWal.Models.Segment{path: \"segment1.wal\", index: 1, buf: \"\", block_count: 0, blocks: :array.new()}\n    %ExWal.Models.Segment{path: \"segment1.wal\", index: 1, buf: \"\", block_count: 0, blocks: :array.new()}","ref":"ExWal.Models.Segment.html#module-examples"},{"type":"module","title":"Struct Fields - ExWal.Models.Segment","doc":"- `path` - The path of the segment file.\n- `index` - The index of the segment.\n- `buf` - The binary buffer of the segment.\n- `block_count` - The count of blocks within the segment.\n- `blocks` - The array of blocks within the segment.","ref":"ExWal.Models.Segment.html#module-struct-fields"},{"type":"type","title":"ExWal.Models.Segment.t/0","doc":"","ref":"ExWal.Models.Segment.html#t:t/0"},{"type":"behaviour","title":"ExWal.Store","doc":"storage behavior","ref":"ExWal.Store.html"},{"type":"function","title":"ExWal.Store.append/3","doc":"","ref":"ExWal.Store.html#append/3"},{"type":"callback","title":"ExWal.Store.append/3","doc":"","ref":"ExWal.Store.html#c:append/3"},{"type":"function","title":"ExWal.Store.close/2","doc":"","ref":"ExWal.Store.html#close/2"},{"type":"callback","title":"ExWal.Store.close/2","doc":"","ref":"ExWal.Store.html#c:close/2"},{"type":"function","title":"ExWal.Store.mkdir/3","doc":"","ref":"ExWal.Store.html#mkdir/3"},{"type":"callback","title":"ExWal.Store.mkdir/3","doc":"","ref":"ExWal.Store.html#c:mkdir/3"},{"type":"callback","title":"ExWal.Store.new/1","doc":"","ref":"ExWal.Store.html#c:new/1"},{"type":"function","title":"ExWal.Store.open/4","doc":"","ref":"ExWal.Store.html#open/4"},{"type":"callback","title":"ExWal.Store.open/4","doc":"","ref":"ExWal.Store.html#c:open/4"},{"type":"function","title":"ExWal.Store.read_all/2","doc":"","ref":"ExWal.Store.html#read_all/2"},{"type":"callback","title":"ExWal.Store.read_all/2","doc":"","ref":"ExWal.Store.html#c:read_all/2"},{"type":"function","title":"ExWal.Store.rename/3","doc":"","ref":"ExWal.Store.html#rename/3"},{"type":"callback","title":"ExWal.Store.rename/3","doc":"","ref":"ExWal.Store.html#c:rename/3"},{"type":"function","title":"ExWal.Store.rm/2","doc":"","ref":"ExWal.Store.html#rm/2"},{"type":"callback","title":"ExWal.Store.rm/2","doc":"","ref":"ExWal.Store.html#c:rm/2"},{"type":"function","title":"ExWal.Store.sync/2","doc":"","ref":"ExWal.Store.html#sync/2"},{"type":"callback","title":"ExWal.Store.sync/2","doc":"","ref":"ExWal.Store.html#c:sync/2"},{"type":"function","title":"ExWal.Store.write_all/3","doc":"","ref":"ExWal.Store.html#write_all/3"},{"type":"callback","title":"ExWal.Store.write_all/3","doc":"","ref":"ExWal.Store.html#c:write_all/3"},{"type":"type","title":"ExWal.Store.handler/0","doc":"","ref":"ExWal.Store.html#t:handler/0"},{"type":"type","title":"ExWal.Store.mode/0","doc":"","ref":"ExWal.Store.html#t:mode/0"},{"type":"type","title":"ExWal.Store.path/0","doc":"","ref":"ExWal.Store.html#t:path/0"},{"type":"type","title":"ExWal.Store.permision/0","doc":"","ref":"ExWal.Store.html#t:permision/0"},{"type":"type","title":"ExWal.Store.t/0","doc":"","ref":"ExWal.Store.html#t:t/0"},{"type":"module","title":"ExWal.Store.File","doc":"file storage","ref":"ExWal.Store.File.html"},{"type":"function","title":"ExWal.Store.File.append/3","doc":"","ref":"ExWal.Store.File.html#append/3"},{"type":"function","title":"ExWal.Store.File.close/2","doc":"","ref":"ExWal.Store.File.html#close/2"},{"type":"function","title":"ExWal.Store.File.mkdir/3","doc":"","ref":"ExWal.Store.File.html#mkdir/3"},{"type":"function","title":"ExWal.Store.File.new/1","doc":"","ref":"ExWal.Store.File.html#new/1"},{"type":"function","title":"ExWal.Store.File.open/4","doc":"","ref":"ExWal.Store.File.html#open/4"},{"type":"function","title":"ExWal.Store.File.read_all/2","doc":"","ref":"ExWal.Store.File.html#read_all/2"},{"type":"function","title":"ExWal.Store.File.rename/3","doc":"","ref":"ExWal.Store.File.html#rename/3"},{"type":"function","title":"ExWal.Store.File.rm/2","doc":"","ref":"ExWal.Store.File.html#rm/2"},{"type":"function","title":"ExWal.Store.File.sync/2","doc":"","ref":"ExWal.Store.File.html#sync/2"},{"type":"function","title":"ExWal.Store.File.write_all/3","doc":"","ref":"ExWal.Store.File.html#write_all/3"},{"type":"type","title":"ExWal.Store.File.t/0","doc":"","ref":"ExWal.Store.File.html#t:t/0"},{"type":"module","title":"ExWal.Uvarint","doc":"The Uvarint module is used to read variable-length encoded unsigned integers.\nThe implementation is based on the variable-length encoding specification of Protocol Buffers.\nVariable-length encoding is a method of encoding smaller integers with fewer bytes, thereby saving storage space.\n\nIn the implementation of the Uvarint function, if the highest bit of a byte is 0, then that byte is the last byte of the encoding;\nif the highest bit is 1, then that byte is not the last byte and there are other bytes following it.\nThis encoding method can be used to store any size of unsigned integer, but for larger integers, multiple bytes may be required for encoding.","ref":"ExWal.Uvarint.html"},{"type":"function","title":"ExWal.Uvarint.decode/1","doc":"Decodes a variable-length binary representation into a non-negative integer.","ref":"ExWal.Uvarint.html#decode/1"},{"type":"function","title":"Examples - ExWal.Uvarint.decode/1","doc":"iex> ExWal.Uvarint.decode(<<42::size(8)>>)\n    {42, 1, \"\"}\n\n    iex> ExWal.Uvarint.decode(<<0x8E, 0x02>> <> \"Hello World\")\n    {270, 2, \"Hello World\"}","ref":"ExWal.Uvarint.html#decode/1-examples"},{"type":"function","title":"ExWal.Uvarint.encode/1","doc":"Encodes a non-negative integer into a variable-length binary representation.","ref":"ExWal.Uvarint.html#encode/1"},{"type":"function","title":"Examples - ExWal.Uvarint.encode/1","doc":"iex> ExWal.Uvarint.encode(42)\n    <<42::size(8)>>\n\n    iex> ExWal.Uvarint.encode(300)\n    <<0x8C, 0x02>>","ref":"ExWal.Uvarint.html#encode/1-examples"}],"content_type":"text/markdown"}